%global _MM_MsgDataset;
%let _MM_MsgDataset = sashelp.mdlmgr;

/*
   scoreout  = full   data set
   scoreinput = subset data set
   output = output data set containing variables
*/ 
%macro extraVars( input=, out=, output=);
   %let dsid = %sysfunc(open(&input));
   %let nvars = %sysfunc(attrn(&dsid, NVARS));
   data &output;
      merge &input &out;
      drop 
      %do i=1 %to &nvars;
         %let var1 = %sysfunc(varname(&dsid, &i));
         &var1
      %end;
      ;
   run;
   %let dsid = %sysfunc(close(&dsid)); 
%mend extraVars;

/*******************************************************************
 * genVarMeta - generates variable meta data from a data set
 ********************************************************************/
%Macro genVarMeta(outTable, inTable, keepvars=, defaultRole=' ');
                                                                                                                                                              
   DATA _MMTEMP (KEEP= _MMName _MMType _MMLength _MMLabel _MMFormat _MMRole _MMLevel);
      IF _N_ = 0 THEN DO;
	 SET &inTable (obs=0);
      END;
      RETAIN cnt 0;
      LENGTH _MMNAME $ 32;
      LENGTH _MMTYPE $ 1;
      LENGTH _MMLENGTH $ 8;
      LENGTH _MMLABEL $ 256;
      LENGTH _MMFORMAT $32;
      LENGTH _MMLEVEL $32;
      LENGTH _MMROLE $ 32;

      length msg $ 256; 

      %let keepvars = %upcase(&keepvars);

      dsid=OPEN("&inTable","i");
      IF (dsid ^= 0) THEN DO;
         numvars= attrn(dsid,"nvars");
         IF ("&_MMTRACE" = "1") THEN
         do;
            msg = sasmsg("&_MM_MsgDataset", "genvarmacro_genVarMeta_max_vars_note", "noquote", strip(numvars));
            putlog msg;
         end;
         DO i=1 TO numvars;
            _MMName = varname(dsid,i);
            vtype = vartype(dsid,i);
            /* convert type to character indicator */
            IF vtype = '1' or vtype = 'N' THEN _MMType = 'N';
            ELSE _MMType = 'C';
            _MMLength = strip(varlen(dsid,i));
            _MMLabel = varlabel(dsid,i);
            _MMFormat = vformatx(_MMName);
            _MMLevel= "";
            %if (&defaultRole ^= ) %then %do;
            _MMRole = &defaultRole;
            %end;
            %else %do;
            _MMRole = ' ';
            %end;

            %if (&keepvars = ) %then
            %do;
            IF ("&_MMTRACE" = "1") THEN
            do;
               msg = sasmsg("&_MM_MsgDataset", "genvarmacro_genVarMeta_name_type_note", "noquote", strip(_MMname), strip(_MMtype));
               putlog msg;
            end;
            OUTPUT;
            cnt = cnt +1;
            %end;
            %else %do;
            IF indexw("&keepvars", Upcase(Strip(_MMName))) THEN DO;
               IF ("&_MMTRACE" = "1") THEN
               do;
                  msg = sasmsg("&_MM_MsgDataset", "genvarmacro_genVarMeta_keeping_name_type_note", "noquote", strip(_MMname), strip(_MMtype));
                  putlog msg;
               end;
               OUTPUT;
               cnt = cnt +1;
            END;
            %end;
            ;
         END;
         IF ("&_MMTRACE" = "1") THEN
         do;
            msg = sasmsg("&_MM_MsgDataset", "genvarmacro_genVarMeta_variable_count_note", "noquote", strip(cnt));
            putlog msg;
         end;
      END;
      ELSE DO;
         emsg = SYSMSG();
         PUT emsg;
      END;
      STOP;
   Run;
	 DATA &outTable (KEEP= Name Type Length Label Format Role Level);
	    set _MMTEMP;
	    NAME=_MMNAME;
		TYPE=_MMTYPE;
		LENGTH=_MMLength;
		LABEL=_MMLabel;
		FORMAT=_MMFormat;
		ROLE=_MMRole;
		LEVEL=_mmLevel;
	 Run;

%Mend genVarMeta;

/*******************************************************************
 * genVarXML macro generates SMM XML from variable meta data table
 * Note: check the existence of the filename passed as XMLFref
 * before calling genVarXML.
 ********************************************************************/
%Macro genVarXML(metaTbl, XMLFref);
   LIBNAME  &XMLFref XML tagset=tagsets.sasxmog xmlencoding="UTF-8";
   DATA &XMLFref..variable;
      SET &metaTbl;
      OUTPUT;
      ; 
   RUN;

   LIBNAME   &XMLFref clear;
%Mend genVarXML;

%Macro genVarMacro(scoreinput=smmmodel.modelinput, scoreoutput=SMMModel.modeloutput, target=smmmodel.target);
 %if %sysfunc(exist(&scoreinput)) %then %do;
  %genvarMeta(modelinput, &scoreinput, defaultRole='Input')
  %genVarXML(modelinput, input)
 %end;
 %if %sysfunc(exist(&scoreoutput)) %then %do;
	%if %sysfunc(exist(&scoreinput)) %then %do;
  		%extraVars(input=&scoreinput, out=&scoreoutput, output=tempout)
	%end;
	%else %do;
		data tempout;
			set &scoreoutput;
		run;
	%end;
  %genvarMeta(modeloutput, tempout, defaultRole='Output')
  %genVarXML(modeloutput, output)
 %end;
 %if %sysfunc(exist(&target)) %then %do;
  %genvarMeta(modeltarget, &target, defaultRole='Target')
  %genVarXML(modeltarget, target)
 %end;
%Mend genVarMacro;
